@@ -3,7 +3,7 @@
 import numpy as np
 import numpy.linalg as la
 import matplotlib.pyplot as plt
-from .utils import utils as ut
+from . import utils as ut
 from functools import reduce, lru_cache
 from sympy.physics.wigner import wigner_3j, wigner_6j
 
@@ -95,6 +95,7 @@ def get_ryabov_a_coeffs(k_max):
 def calc_stev_ops(k_max, J, jp, jm, jz):
     """
     Calculates all Stevens operators Okq with k even and odd up to k_max
+    k_max must be <= 12
 
     Positional arguments:
         k_max (int)     : maximum k value
@@ -112,6 +113,9 @@ def calc_stev_ops(k_max, J, jp, jm, jz):
                                           k=1 q=-k->k, k=2 q=-k->k ...
     """
 
+    # Only k <= 12 possible at double precision
+    k_max = min(k_max, 12)
+
     # Get a(k,q,m,i) coefficients and greatest common factors
     a, f = get_ryabov_a_coeffs(k_max)
 
@@ -240,20 +244,28 @@ def load_CFPs(f_name, style="phi"):
     return CFPs
 
 
-def calc_HCF(J, CFPs, stev_ops, k_max=False):
+def calc_HCF(J, CFPs, stev_ops, k_max=False, oef=False):
     """
     Calculates crystal field Hamiltonian (HCF) using CFPs and Stevens operators
     Okq, where k even and ranges 2 -> 2j
 
+    Hamiltonian is sum_k sum_q Bkq Okq
+
     Positional arguments:
-        J (float)        : J quantum number
-        CFPs (np.array)  : Matrix representation of angular momentum operator
-        stevops (list of np.arrays) : List containing each stevens operator
-                                        indexed k=2 q=-k->k, k=4 q=-k->k ...
+        J (float)                   : J quantum number
+        CFPs (np.array)             : Even k crystal Field parameters in order
+                                      k=2 q=-k->k, k=4 q=-k->k ...
+        stevops (list of np.arrays) : List containing each even k
+                                      stevens operator
+                                      indexed [k_ind,q_ind,:,:]
+                                      k_ind = 1, .. k_max/2
+                                      q_ind = 1, .. 2*q_max + 5 :(q_max=+-k)
 
     Keyword arguments:
-        k_max (int) : Maximum value of k to use in summation - by default 2*J
-
+        k_max (int)    : Maximum value of k to use in summation - by default 2*J
+        oef (np.array) : Operator equivalent factors for each CFP
+                         i.e. 27 CFPs = 27 OEFs
+                         by default these are set to unity
     Returns:
         HCF (np.array)    : Matrix representation of Crystal Field Hamiltonian
         CF_val (np.array) : Eigenvalues of HCF (lowest eigenvalue is zero)
@@ -264,17 +276,20 @@ def calc_HCF(J, CFPs, stev_ops, k_max=False):
     if not k_max:
         k_max = int(2*J)
         k_max -= k_max % 2
+        k_max = min(k_max, 12)
+
+    if not oef:
+        oef = np.ones(np.size(CFPs))
 
     # calculate number of states
     n_states = int(2 * J + 1)
 
     # Form Hamiltonian
-    # No operator equivalent factors are needed as they are included in
-    # the single_aniso CFPs
     HCF = np.zeros([n_states, n_states], dtype=np.complex128)
     for kit, k in enumerate(range(2, k_max+1, 2)):
         for qit, q in enumerate(range(-k, k+1)):
-            HCF += stev_ops[kit, qit, :, :] * CFPs[ut._even_kq_to_num(k, q)]
+            HCF += stev_ops[kit, qit, :, :] * CFPs[ut._even_kq_to_num(k, q)] \
+                    * oef[ut._even_kq_to_num(k, q)]
 
     # Diagonalise
     CF_val, CF_vec = la.eigh(HCF)
@@ -589,6 +604,7 @@ def barrier_figure(J, energies, Jz_expect, trans=False, ax_in=False,
     if not ax_in:
         fig, ax = plt.subplots(1, 1, sharey='all', figsize=figsize)
     else:
+        fig = None
         ax = ax_in
 
     if yax2:
@@ -605,7 +621,7 @@ def barrier_figure(J, energies, Jz_expect, trans=False, ax_in=False,
         markersize='25',
         mew='2.5',
         linewidth=0,
-        color='black',
+        color=state_colour,
         label=levels_name
         )
 
@@ -711,3 +727,51 @@ def barrier_figure(J, energies, Jz_expect, trans=False, ax_in=False,
             plt.show()
 
     return fig, axes
+
+
+def hunds_ground_term(n_elec, n_orb):
+    """
+    Calculate J, L, and S quantum numbers using number of
+    electrons and orbitals
+
+    Positional Arguments:
+        n_elec (int) : Number of electrons
+        n_orb (int)  : Number of orbitals (either 3, 5, or 7 i.e. (p, d, f))
+
+    Returns:
+        j (float) : Total angular momentum quantum number
+        l (float) : Orbital angular momentum quantum number
+        s (float) : Spin angular momentum quantum number
+    """
+
+    # Set constants for given shell
+    if n_orb == 7:
+        ml_vals = [3, 2, 1, 0, -1, -2, -3]
+        max_s = 3.5
+    elif n_orb == 5:
+        ml_vals = [2, 1, 0, -1, -2]
+        max_s = 2.5
+    elif n_orb == 3:
+        ml_vals = [1, 0, -1]
+        max_s = 1.5
+    else:
+        print('Unsupported number of orbitals: {:d}'.format(n_orb))
+        exit()
+
+    # More than half filled
+    if n_elec > n_orb:
+        s = max_s - float(n_elec - n_orb) * 0.5
+        l = float(sum(ml_vals[:n_elec - n_orb]))
+        j = l + s
+    # Less than half filled
+    elif n_elec < n_orb:
+        s = 0.5 * float(n_elec)
+        l = float(sum(ml_vals[:n_elec]))
+        j = l - s
+    # Half filled
+    elif n_elec == n_orb:
+        s = max_s
+        l = 0.
+        j  = 0.
+
+    return j, l, s
